SuperStrict

Rem
	bbdoc: Tiny INI Module
	about: <font face="monospace" size="+1"><b>Lizenz/License:</b> MIT License<br><br>
	In the following text this library is called "this program":<br><br>
	
	(c) 2010 Christian Fiedler

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
	
	You should have received a copy of the GNU Lesser General Public License<br>
	along with this program.  If not, see <http://www.gnu.org/licenses/>.<br><br>
	<b>Beschreibung/Description:</b><br><br>
	Diese Bibliothek enthlt einen kleinen INI-Parser. Einerseits ist sie ziemlich einfach verwendbar,
	aber andererseits gibt es auch gewisse Einschrnkungen.<br>
	Der INI-Parser ist nicht case-sensitive!
	<br><br>
	This library contains a small ini parser. On the one hand it's super simple to use, but on the
	other hand it provides some resrictions.<br>
	This INI-Parser isn't case-sensitive!<br>
	<b>As my English isn't very good, I can't guarantee that the English translation is absolutely correct!</b>
	</font>
EndRem
Module thunder.ini

ModuleInfo "Version: 0.80 "
ModuleInfo "Author: Christian Fiedler"
ModuleInfo "License: MIT License"
ModuleInfo "Copyright: Christian Fiedler"
ModuleInfo "Modserver: -"

ModuleInfo "History: Version 0.80"
ModuleInfo "History: (Ziemlich groer Sprung) Funktionen, die Sektoren verarbeiten, GetBool und"
ModuleInfo "History: AddINIEntry hinzugefgt."
ModuleInfo "History: Auerdem alle Funktionen berarbeitet - brl.retro ist nicht notwendig"
ModuleInfo "History: Version 0.05"
ModuleInfo "History: Erste Funktionen eingefgt"

Import brl.filesystem
Import brl.linkedlist

Private

Global entrylist:TList=CreateList()

Type TINIEntry
	Field name$,wert$,isparent%
	Function Create(name$,wert$,isparent%,link:TLink=Null)
		Local z:TINIEntry=New TINIEntry
		z.name=name
		z.wert=wert
		z.isparent=isparent
		If link=Null Then
			entrylist.addLast(z)
		Else
			entrylist.insertBeforeLink(z,link)
		EndIf
	EndFunction
EndType

Public

Rem
bbdoc: Ldt alle Eintrge einer INI-Datei in den Arbeitsspeicher. Dort knnen sie mit anderen Funktionen des Moduls modifiziert werden<br>Loads all entries of @file to RAM. They can be modified by other functions of this module
returns: False, wenn @file nicht existiert oder nicht geffnet werden kann.False if @file does not exist or cannot be opened
End Rem
Function LoadINI%(file$)
	'Achtung! Lscht eingelesene Daten einer anderen INI (zur Sicherheit)
	If Not entrylist.isempty() Then entrylist.clear()
	Local name$,wert$,parent$="main"
	If FileType(file)<>1 And FileType(file+".ini")=1 Then
		file:+".ini"
	ElseIf FileType(file)<>1 Then 
		Return False
	EndIf
	Local datei:TStream=ReadFile(file)
	If datei=Null Then Return False
	While Not Eof(datei)
		name=ReadLine(datei).Trim()
		If name.startsWith("[") And name.endswith("]") Then
			name=name[1..(name.length-1)].Trim()
			TINIEntry.Create(name,"",1)
		ElseIf name.startsWith(";") Then
			'Kommentar
			Continue
		Else
			wert=name[(name.find("=")+1)..]
			name=name[..(name.find("="))]
			TINIEntry.Create(name,wert,0)
		EndIf
	Wend
	Return True
EndFunction

Rem
bbdoc: Konvertiert die Wrter 'yes' bzw 'no' in boolsche Werte.<br>Converts the words 'yes' and 'no' respectively, into boolean values.
returns: Wenn @w weder 'yes' noch 'no' ist, ist der Rckgabewert -1. Wenn @w 'yes' ist, ist der Rckgabewert 1, ansonsten 0.<br>If @w is neither 'yes' nor 'no' the function will return -1. If it is 'yes', the function will return 1 and if it's false, you'll get 0
EndRem
Function GetBool%(w$)
	w=(w.Tolower()).Trim()
	Return (w<>"yes" And w<>"no")*-1+(w="yes")
EndFunction

Rem
bbdoc: Liest den Wert der Variablen @name aus der geladenen INI-Datei <br>Reads the value of a variable @name in the currently loaded ini file
returns: Den Wert der Variable @name. Wenn die Variable nicht existiert, Null<br>Value of variable @name. If this variable does not exist, the function returns Null
End Rem
Function ReadINIEntry$(name$,parent$="main") 
	Local cparent$="main"
	For Local z:TINIEntry=EachIn entrylist
		If z.isparent Then
			cparent=z.name
		ElseIf z.name.ToLower()=name.ToLower() And cparent.toLower()=parent.toLower() Then
			Return z.wert
		EndIf
	Next
	Return Null
EndFunction

Rem
bbdoc: Lscht den Eintrag @name aus dem Arbeitsspeicher<br>Deletes a specified entry @name from RAM
returns: Wenn die Variable @name nicht existiert, gibt die Funktion False zurck, ansonsten True<br>If there's no variable @name, this function will return False
End Rem
Function DeleteINIEntry%(name$,parent$="main")
	Local cparent$="main"
	For Local z:TINIEntry=EachIn entrylist
		If z.isparent Then
			cparent=z.name
		ElseIf z.name.ToLower()=name.ToLower() And cparent.toLower()=parent.toLower() Then
			entrylist.remove(z)
			Return True
		EndIf
	Next
	Return False
EndFunction

Rem
bbdoc: Modifiziert den Wert eines existierenden Eintrags @name<br>Modifies the value of an existing entry @name
returns: Wenn die Variable @name nicht existiert gibt die Funktion False zurck<br>If there's no variable @name, the function returns False
End Rem
Function ModifyINIEntry%(name$,wert$,parent$="main")
	Local cparent$="main"
	For Local z:TINIEntry=EachIn entrylist
		If z.isparent Then
			If cparent=parent Then Return False
			cparent=z.name
		ElseIf z.name.ToLower()=name.ToLower() And cparent.toLower()=parent.toLower() Then
			z.wert=wert
			Return True
		EndIf
	Next
	Return False
EndFunction

Rem
bbdoc: Modifiziert den Wert eines existierenden Eintrags @name. Falls dieser nicht existiert, wird eine solche erstellt<br>Modifies the value of an existing entry @name. if it does not exist, it will be created
End Rem
Function WriteINIEntry(name$,wert$,parent$="main")
	Local cparent$="main"
	For Local z:TINIEntry=EachIn entrylist
		If z.isparent Then
			If cparent.toLower()=parent.toLower() Then
				TINIEntry.Create(name,wert,0,entrylist.FindLink(z))
				Return
			EndIf
			cparent=z.name
		ElseIf z.name.ToLower()=name.ToLower() And cparent.toLower()=parent.toLower() Then
			z.wert=wert
			Return
		EndIf
	Next
	TINIEntry.Create(name,wert,0)
EndFunction

Rem
bbdoc: Erstellt einen Eintrag und stellt ihn in den letzten Sektor der INI-Datei<br>Adds an entry to the inis last section
End Rem
Function AddINIEntry(name$,wert$)
	TINIEntry.Create(name,wert,0)
EndFunction

Rem
bbdoc: Schreibt die jetztige Version der INI in eine Datei<br>Writes the current modification of the intern list into an ini file
returns: Gibt False zurck, wenn @file nicht erstellt werden konnte<br>False if @file could not be created
End Rem
Function SaveINI%(file$)
	'Lscht Daten der INI nicht!
	Local datei:TStream=WriteFile(file)
	If datei=Null Then Return 0
	For Local z:TINIEntry=EachIn entrylist
		If z.isparent Then
			WriteLine datei,"["+z.name+"]"
		Else
			WriteLine datei,z.name+"="+z.wert
		EndIf
	Next
	CloseFile datei
	Return 1
EndFunction

Rem
bbdoc: Lscht alle Eintrge, die in den Arbeitsspeicher geladen wurden<br>Clears all entries, which were loaded into memory
EndRem
Function ClearINI()
	entrylist.clear()
EndFunction

Rem
bbdoc: Erstellt einen Sektor @name und hngt ihn hinten an<br>Adds a section @name to the ini
returns: Wenn der Sektor @name existiert, gibt die Funktion False zurck<br>If a section called @name already exists, the function returns False
about: Kann zum programminternen Erstellen einer INI gemeinsam mit #AddINIEntry ntzlich sein<br>Both this and #AddINIEntry function can be useful for creating an ini file.
EndRem
Function AddINISection%(name$)
	For Local z:TINIEntry=EachIn entrylist
		If z.isparent And z.name.tolower()=name.tolower() Then Return False
	Next
	TINIEntry.Create(name,"",1)
	Return True
EndFunction

Rem
bbdoc: Benennt einen Sektor @name der INI-Datei um<br>Renames a section in the currently loaded ini file
returns: Wenn der Sektor @name nicht existiert, gibt die Funktion False zurck<br>If there's no section @name, the function returns False
EndRem
Function RenameINISection%(name$,newname$)
	For Local z:TINIEntry=EachIn entrylist
		If z.name.toLower()=name.toLower() And z.isparent Then z.name=newname;Return True
	Next
	Return False
EndFunction

Rem
bbdoc: Bennent einen Sektor in der INI-Datei um. Falls der Sektor @name nicht existiert, wird ein Sektor @newname erstellt<br>Renames a section in the currently loaded ini file. If there's no section @name, a section named @newname will be created anyway
EndRem
Function WriteINISection(name$,newname$)
	For Local z:TINIEntry=EachIn entrylist
		If z.name.toLower()=name.toLower() And z.isparent Then z.name=newname;Return
	Next
	AddINISection(name)
EndFunction

Rem
bbdoc: Lscht einen Sektor der INI-Datei<br>Deletes a section in the currently loaded ini file
returns: Wenn der Sektor @name nicht existiert, gibt die Funktion False zurck<br>If there's no section @name, the function returns False
EndRem
Function DeleteINISection%(name$)
	Local begin%=0
	For Local z:TINIEntry=EachIn entrylist
		If begin=1 And z.isparent Then Return True
		If z.name.toLower()=name.toLower() And z.isparent And begin=0 Then begin=1
		If begin=1 Then entrylist.remove(z)
	Next
	Return False
EndFunction